@@ -16,7 +16,8 @@ from collections import OrderedDict
 from collections.abc import Sized
 
 import pandas as pd
-from transformers import TrainerCallback
+from torch.utils.tensorboard import SummaryWriter
+from transformers import Trainer, TrainerCallback
 import sty
 import colorama
 from tqdm.auto import tqdm
@@ -32,7 +33,7 @@ __all__ = [
     'hex2rgb', 'MyTheme', 'MyFormatter', 'get_logger',
     'CheckArg', 'ca',
     'now',
-    'MlPrettier', 'MyProgressCallback'
+    'MlPrettier', 'MyProgressCallback', 'LogStep'
 ]
 
 
@@ -408,7 +409,7 @@ class MlPrettier:
 
     def __init__(
             self, ref: Dict[str, Any] = None, metric_keys: List[str] = None, no_prefix: Iterable[str] = no_prefix,
-            with_color: bool = True
+            with_color: bool = False
     ):
         """
         :param ref: Reference that are potentially needed
@@ -499,6 +500,7 @@ class MyProgressCallback(TrainerCallback):
         """
         self.training_bar = None
         self.prediction_bar = None
+
         self.train_only = train_only
         self.step_per_epoch = None
         self.current_step = None
@@ -514,7 +516,7 @@ class MyProgressCallback(TrainerCallback):
         if not is_eval:  # heuristic judging by the eval #epoch shown
             n_ep += 1
 
-        return MlPrettier(ref=dict(epoch=state.num_train_epochs))('epoch', n_ep)
+        return MlPrettier(ref=dict(epoch=state.num_train_epochs), with_color=True)('epoch', n_ep)
 
     def on_epoch_begin(self, args, state, control, **kwargs):
         if state.is_local_process_zero:
@@ -561,6 +563,63 @@ class MyProgressCallback(TrainerCallback):
     def on_train_end(self, args, state, control, **kwargs):
         pass
 
+    @staticmethod
+    def get_current_progress_bar(trainer: Trainer):
+        """
+        Intended for adding per-step metrics to the progress bar during HF training
+
+        This is a hack,
+            since HF API don't support per-step callback not to mention exposing those metrics to the progress bar
+        """
+        callback = next(cb for cb in trainer.callback_handler.callbacks if isinstance(cb, MyProgressCallback))
+        return callback.training_bar if trainer.model.training else callback.prediction_bar
+
+
+class LogStep:
+    """
+    My typical terminal, file-write, tqdm logging for a single step
+    """
+    def __init__(
+            self, trainer: Trainer = None, prettier: MlPrettier = None,
+            logger: logging.Logger = None, file_logger: logging.Logger = None,
+            tb_writer: SummaryWriter = None
+    ):
+        if not trainer:
+            raise ValueError(f'A {pl.i("trainer")} is required')
+        if not hasattr(trainer, 'with_tqdm'):
+            raise ValueError(f'Trainer ill-formed: A custom {pl.i("with_tqdm")} field is needed')
+        self.trainer = trainer
+        self.prettier = prettier
+        self.logger = logger
+        self.file_logger = file_logger
+        self.tb_writer = tb_writer
+
+    def _should_add(self, key: str) -> bool:
+        return self.prettier.should_add_split_prefix(key) if self.prettier else True
+
+    def _call__(self, d_log: Dict, training: bool = None):
+        training = training or self.trainer.model.training
+        d_log_write = self.prettier(d_log) if self.prettier else d_log
+
+        if self.tb_writer:
+            tb_step = d_log.get('step') if training else d_log.get('epoch')
+            pref = 'train' if training else 'eval'
+            for k, v in d_log.items():
+                if self._should_add(k):
+                    self.tb_writer.add_scalar(tag=f'{pref}/{k}', scalar_value=v, global_step=tb_step)
+
+        if self.trainer.with_tqdm:  # a custom field I added
+            pbar = MyProgressCallback.get_current_progress_bar(self.trainer)
+            if pbar:
+                tqdm_kws = {k: pl.i(v) for k, v in d_log_write.items() if self._should_add(k)}
+                pbar.set_postfix(tqdm_kws)
+        else:
+            if self.logger:
+                self.logger.info(pl.i(d_log))
+
+        if self.file_logger:
+            self.file_logger.info(pl.nc(d_log))
+
 
 class CheckArg:
     """
